#!/usr/bin/python

import os
import sys
import subprocess
import semver


MIN_SEM_VER = '0.32.2'
MIGRATE_SCRIPTS_DIRPATH = '/opt/edge/migrations'


def log_info(log_text):
    print log_text

if len(sys.argv) == 1:
    log_info('clean installation')
    sys.exit(0)

elif len(sys.argv) == 2:
    last_dch_ver = sys.argv[1]
    last_sem_ver = last_dch_ver.split('-')[0]

else:
    log_info('invalid args')
    sys.exit(1)

if semver.compare(last_sem_ver, MIN_SEM_VER) < 0:
    log_info('version < %s, migrate configs manually' % MIN_SEM_VER)
    sys.exit(0)

# define migration-plan
migration_dirs = os.listdir(MIGRATE_SCRIPTS_DIRPATH)
migration_plan = []
for sem_ver in migration_dirs:
    if sem_ver == 'next':
        continue

    if semver.compare(sem_ver, last_sem_ver) > 0:
        migration_plan.append(sem_ver)

migration_plan.sort(semver.compare)
log_info('migration-plan: %s' % migration_plan)

for sem_ver in migration_plan:
    print "migrate to %s" % sem_ver
    migration_dirpath = os.path.join(MIGRATE_SCRIPTS_DIRPATH, sem_ver)
    runparts = subprocess.Popen(
        '. /opt/edge/rc.setenv; /bin/run-parts ' + migration_dirpath,
        stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

    stdout, stderr = runparts.communicate()
    if runparts.returncode:
        log_info('FAIL: ' + stderr)
        sys.exit(1)

log_info('DONE')
