from twisted.internet import reactor

from _args import KernelArgs
import edge.eventlog as eventlog

from _db import Db
import edge.db as edge_db
from _react import ReactProducer
from _event import EventConsumer, StubEventConsumer
import _counters

import _models


class BaseKernel:
    def __init__(self, args_db, react_producer, event_consumer):
        self._args_db = args_db
        self._react_producer = react_producer
        self._react_producer.set_fact_handler(self._handle_react)
        self._event_consumer = event_consumer

    def _handle_react(self, fact):
        """
        :type fact: _models.Fact
        """
        # noinspection PyBroadException
        try:
            platear_id, media_id = self._args_db.add_fact(fact)
            self._event_consumer.consume_fact(fact, platear_id, media_id)
        except edge_db.DbError:
            pass
        except:
            eventlog.log_debug_last_exc()


class Kernel(BaseKernel):
    def __init__(self, twisted_reactor, args):
        """
        :type args: KernelArgs
        """
        counters = _counters.StatCounters(
            reactor, args.counters,
            args.event_consumer is not None)

        db = Db.create(twisted_reactor, counters.db, args.db, args.db_post, args.db_chan)

        react_producer = ReactProducer(twisted_reactor, counters.react_producer, args.react_producer)

        event_consumer_ctor = EventConsumer if args.event_consumer else StubEventConsumer
        event_consumer = event_consumer_ctor(twisted_reactor, counters.event_consumer, args.event_consumer)

        BaseKernel.__init__(self, db, react_producer, event_consumer)
        self._twisted_reactor = twisted_reactor
        self._counters = counters

    def run(self):
        self._twisted_reactor.run()
