
from edge.queue import JsonFileQueue, ImageFileQueue
from _kernel_param_set import SceneEventArgs, ImageStoreArgs


class SceneEventConsumer:
    _STATUS_SUCCESS = 1
    _STATUS_FAILURE = 2

    def __init__(self, twisted_reactor, codec, json_file_queue_list, image_file_queue):
        """
        :type codec: str
        :type json_file_queue_list: list[JsonFileQueue]
        :type image_file_queue: ImageFileQueue
        """
        self._twisted_reactor = twisted_reactor
        self._target_codec = codec
        self._json_file_queue_list = json_file_queue_list
        self._image_file_queue = image_file_queue

    def consume_scene(self, scene_ts, rgba_image):
        """
        :type scene_ts: int
        :type rgba_image: PIL.Image.Image
        """
        image_width, image_height = rgba_image.size
        image_ref = self._image_file_queue.add_image(scene_ts, rgba_image)

        event_args_jnode = {
            'Ts': image_ref.ts,
            'Status': SceneEventConsumer._STATUS_SUCCESS,  # todo
            'SizeWh': [image_width, image_height],
            'Url': image_ref.url,
            'Codec': image_ref.codec
        }

        map(lambda json_file_queue: json_file_queue.add_jnode(event_args_jnode),
            self._json_file_queue_list)

    @staticmethod
    def create(twisted_reactor, scene_args, store_args):
        """
        :type twisted_reactor: twisted.internet.interfaces.IReactorTime
        :type scene_args: SceneEventArgs
        :type store_args: ImageStoreArgs
        """
        json_file_queue_list = map(
            lambda json_file_queue_args: JsonFileQueue(json_file_queue_args), scene_args.json_queue_args_list)

        image_file_queue = ImageFileQueue(
            twisted_reactor, store_args.codec,
            store_args.image_queue_args.endpoint,
            store_args.image_queue_args.min_file_count,
            store_args.image_queue_args.max_file_count,
            scene_args.log_level)

        return SceneEventConsumer(twisted_reactor, store_args.codec, json_file_queue_list, image_file_queue)
