import os
import inspect
import traceback

try:
    import _syslog_logger as module_logger
except ImportError:
    import _stdout_logger as module_logger


def open_log(log_ident, log_level):
    """
    :type log_ident: str
    :type log_level: str
    """
    module_logger.open_log(log_ident, log_level)


def close_log():
    module_logger.close_log()


def _get_final_entry_text(entry_level, entry_text):
    """
    :type entry_level: str
    :type entry_text: str
    :rtype: str
    """
    template = '[{entry_level}|{caller_file}:{caller_line}|{caller_class}::{caller_fn}] {entry_text}'

    caller_stack_frame = inspect.stack()[2][0]
    caller_file = os.path.basename(inspect.getfile(caller_stack_frame))
    caller_line = inspect.getlineno(caller_stack_frame)
    caller_fn = inspect.stack()[2][3]
    try:
        caller_class = caller_stack_frame.f_locals["self"].__class__.__name__
    except KeyError:
        template = '[{entry_level}|{caller_file}:{caller_line}|{caller_fn}] {entry_text}'

    return template.format(**locals())


def log_trace(entry_text):
    """
    :type entry_text: str
    """
    final_entry_text = _get_final_entry_text('t', entry_text)
    module_logger.log_trace(final_entry_text)


def log_debug(entry_text):
    """
    :type entry_text: str
    """
    final_entry_text = _get_final_entry_text('d', entry_text)
    module_logger.log_debug(final_entry_text)


def log_info(entry_text):
    """
    :type entry_text: str
    """
    final_entry_text = '[i] {}'.format(entry_text)
    module_logger.log_info(final_entry_text)


def log_warning(entry_text):
    """
    :type entry_text: str
    """
    final_entry_text = _get_final_entry_text('w', entry_text)
    module_logger.log_warning(final_entry_text)


def log_error(entry_text):
    """
    :type entry_text: str
    """
    final_entry_text = _get_final_entry_text('e', entry_text)
    module_logger.log_error(final_entry_text)


def log_invalid_json_file(exception, json_abspath):

    content = None
    if json_abspath:
        with open(json_abspath) as json_file:
            lines = json_file.readlines()
        content = ''.join(lines).decode('utf-8-sig').encode('utf-8')

    log_error(traceback.format_exc())
    log_error(exception)  # TODO Maybe fix type
    log_error('Invalid JSON data in the file "%s."' % json_abspath)
    log_error('<--BEGIN')
    log_error(content)
    log_error('END-->')


def log_exception(exception):  # TODO Fix this
    """
    :type exception: subclass of Exception
    """
    print exception
