import traceback

import _eventlog
from _event_consumer import *
from _renderer import Renderer
from _lr_color_tout_store import LrColorToutStore

from twisted.python.runtime import platform
from edge.lane_detector import LaneDetector

if platform.isLinux():
    from _inotify_react_producer import INotifyAddSceneReactProducer as AddSceneReactProducer
else:
    from _react_producer import StubAddSceneReactProducer as AddSceneReactProducer


class BaseKernel:
    def __init__(self, react_producer, event_consumer, image_renderer):

        self._react_producer = react_producer
        self._react_producer.set_addscene_react_handler(self.handle_add_scene_react)
        self._event_consumer = event_consumer
        self._image_renderer = image_renderer

        #
        self._last_react_ts = 0

    def handle_add_scene_react(self, tr_scene, rgba_pil_image):
        """
        :type tr_scene: _models.TrScene
        :type rgba_pil_image: PIL.Image.Image
        """
        try:
            react_ts = tr_scene.img_data.image_ts
            if react_ts <= self._last_react_ts:
                return

            self._image_renderer.render_tr_scene(rgba_pil_image, tr_scene)

            self._last_react_ts = react_ts
            self._event_consumer.consume_scene(react_ts, rgba_pil_image)

        except Exception as e:
            _eventlog.log_error(traceback.format_exc())
            _eventlog.log_exception(e)


class Kernel(BaseKernel):
    def __init__(self, twisted_reactor, args):

        lane_detector = LaneDetector(args.lane_detector_args.id_to_lane_list,
                                     args.lane_detector_args.img_width,
                                     args.lane_detector_args.img_height)

        react_producer = AddSceneReactProducer(
            twisted_reactor,
            args.add_scene_react)

        event_consumer = SceneEventConsumer.create(twisted_reactor, args.scene_event, args.image_store)

        color_tout_store = LrColorToutStore(twisted_reactor, args.renderer.lr_store_tout)

        renderer = Renderer(
            args.renderer,
            color_tout_store,
            lane_detector)

        BaseKernel.__init__(self, react_producer, event_consumer, renderer)

        self._twisted_reactor = twisted_reactor

    def run(self):
        self._twisted_reactor.run()
