import os
import sys
import ujson

from twisted.internet import reactor
from edge.argset import ArgSet

import _eventlog
from _kernel_param_set import KernelArgs
from _kernel import Kernel
from __version import VERSION_STRING


def run_module(raw_arg_set):
    """
    :type raw_arg_set: list of str
    """
    arg_set = ArgSet(raw_arg_set)
    if arg_set.mode == ArgSet.MODE_NONE:
        print "The command line's arguments are invalid"
        sys.exit(1)

    if arg_set.mode == ArgSet.MODE_VERSION:
        print VERSION_STRING
        return

    if arg_set.mode == ArgSet.MODE_STARTUP:
        module_conf_name = 'overlayrender.{}'.format(arg_set.module_id)
        _eventlog.open_log(module_conf_name, arg_set.log_level)
        _eventlog.log_info('Startup...')
        try:
            main_conf_abspath = os.path.join('/etc/edge', module_conf_name, 'main.conf')
            with open(main_conf_abspath) as main_conf_jnode:
                main_conf_jnode = ujson.load(main_conf_jnode)

            param_set = KernelArgs.deserialize(main_conf_jnode, arg_set.log_level)
            kernel = Kernel(reactor, param_set)
            kernel.run()

        finally:
            _eventlog.log_info('Exit')
            _eventlog.close_log()

        return
