#!/usr/bin/python
import sys
import json
import os
import edge.eventlog as eventlog
from edge.fs_helpers import create_jnode_from_default
from edge.admin.reply_helpers import *


UI_ACTIONS_JNODE_FILEPATH = '/etc/edge/setupstore/admin/actions/actions.json'
UI_ACTIONS_DEFJNODE_FILEPATH = '/opt/edge/share/setupstore/ui/actions-jnode.json'


def run(http_method):

    # create ui-jnode
    if not os.path.exists(UI_ACTIONS_JNODE_FILEPATH):
        # noinspection PyBroadException
        try:
            create_jnode_from_default(UI_ACTIONS_DEFJNODE_FILEPATH, UI_ACTIONS_JNODE_FILEPATH)

        except:
            eventlog.log_debug_last_exc()
            return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)
    try:
        with open(UI_ACTIONS_JNODE_FILEPATH, 'r') as actions_file:
            actions_jnode = json.load(actions_file)

    except (IOError, ValueError):
        return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)

    # Dispatch http-method.
    if http_method == 'GET':
        response_jnode = reply_done(actions_jnode)

    else:
        eventlog.log_debug('fail: %s not allowed' % http_method)
        response_jnode = reply_fail(REPLY_METHOD_NOT_ALLOWED_405)

    return response_jnode


def _main():
    eventlog.open_log('cgi-actions', 'debug')
    exit_code = 1
    # noinspection PyBroadException
    try:
        http_mode = os.environ['REQUEST_METHOD']
        path_info = os.environ['PATH_INFO']
        if path_info:
            response_dict = reply_fail(REPLY_NOT_FOUND_404)
        else:
            response_dict = run(http_mode)
        print json.dumps(dict(response_dict), separators=(',', ':'))
        exit_code = 0

    except:
        print json.dumps(reply_fail(REPLY_INTERNAL_SERVER_ERROR_500), separators=(',', ':'))

    finally:
        eventlog.close_log()

    sys.exit(exit_code)


if __name__ == '__main__':
    _main()
