import os

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *

import edge.pipeline_helpers as helpers


def es_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    # speed
    if ui_entry_point in [Pipeline.Speed.Ui.EVENTSTORE_0]:
        used_ui_modules.update([Pipeline.Speed.Ui.EVENTSTORE_0])
        used_edge_modules.add(Pipeline.Speed.Edge.EVENTSTORE_0)
        creator_ctor = EsCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Speed.Ui.EVENTSTORE_0])
        creator_ctor = EsCreator.create

    # mv
    elif ui_entry_point in [Pipeline.Mv.Ui.EVENTSTORE_0]:
        used_ui_modules.update([Pipeline.Mv.Ui.EVENTSTORE_0])
        used_edge_modules.add(Pipeline.Mv.Edge.EVENTSTORE_0)
        creator_ctor = EsCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Mv.Ui.EVENTSTORE_0])
        creator_ctor = EsCreator.create

    # rtsp
    elif ui_entry_point in [Pipeline.Rtsp.Ui.EVENTSTORE_0]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.EVENTSTORE_0])
        used_edge_modules.add(Pipeline.Rtsp.Edge.EVENTSTORE_0)
        creator_ctor = EsCreator.create

    elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.EVENTSTORE_0])
        creator_ctor = EsCreator.create

    return creator_ctor


class EsCreator(BaseCreator):
    def __init__(self, id_main_setupstore):
        """
        :type id_main_setupstore: str, dict, dict | None, str
        """
        BaseCreator.__init__(self)
        self._id_main_setupstore = id_main_setupstore

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: MsCreator
        """
        # speed
        if ui_entry_point in [Pipeline.Speed.Ui.PIPELINE, Pipeline.Speed.Ui.EVENTSTORE_0]:
            id_main_setupstore = EsCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = EsCreator(id_main_setupstore)

        # mv
        elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE, Pipeline.Mv.Ui.EVENTSTORE_0]:
            id_main_setupstore = EsCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = EsCreator(id_main_setupstore)

        # rtsp
        elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE, Pipeline.Rtsp.Ui.EVENTSTORE_0]:
            id_main_setupstore = EsCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = EsCreator(id_main_setupstore)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: str, dict, dict | None, str
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point.ptype == PTYPE_SPEED:
            ui_eventstore_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.EVENTSTORE_0]
            edge_identity = Pipeline.Speed.Edge.EVENTSTORE_0

        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            ui_eventstore_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.EVENTSTORE_0]
            edge_identity = Pipeline.Mv.Edge.EVENTSTORE_0

        elif ui_entry_point.ptype == PTYPE_LPR_RTSP:
            ui_eventstore_jnode = ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.EVENTSTORE_0]
            edge_identity = Pipeline.Rtsp.Edge.EVENTSTORE_0

        else:
            raise NotImplemented

        edge_gid = edge_identity.gid
        spool_dir = get_spool_dir(ui_pid)

        # create main.conf
        main_conf_jnode = {
            "base": DB_JSON_FILEPATH,
            "monitoring": {
                "stat-path": join_path(spool_dir, 'zabbix', 'es%s' % edge_gid),
                "stat-tout": 60
            },
            "store": {
                "Channel": ui_eventstore_jnode['rw']['channel']
            },
            "fact-react": join_path(spool_dir, 'sc%s-es%s-fact' % (edge_gid, edge_gid))
        }

        fact_jnode = ui_eventstore_jnode['ro'].get('fact-event')
        if fact_jnode is not None:
            main_conf_jnode['fact-event'] = [
                fact_jnode.get('mode', 1),
                fact_jnode.get('host', '127.0.0.1'),
                fact_jnode.get('port', 3344)
            ]

        # create setupstore.conf
        edge_id = helpers.get_edge_id(edge_group, edge_identity)
        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        return edge_id, main_conf_jnode, setupstore_conf_text

    def serialize(self):
        edge_id, main_conf_jnode, setupstore_conf_text = self._id_main_setupstore
        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_EVENTSTORE, edge_id)

        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        main_conf_filepath = join_path(edge_jnode_dir, 'main.conf')
        with open(main_conf_filepath, 'w') as json_file:
            json.dump(main_conf_jnode, json_file, indent=2)

        setupstore_conf_filepath = join_path(edge_jnode_dir, 'setupstore.conf')
        with open(setupstore_conf_filepath, 'w') as text_file:
            text_file.write(setupstore_conf_text)


class EsDeleter:
    def __init__(self, edge_es_dirpath):
        """
        :type edge_es_dirpath: str
        """
        self._edge_es_dirpath = edge_es_dirpath

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: EsDeleter | None
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.EVENTSTORE_0)
            edge_type = Pipeline.Rtsp.Edge.EVENTSTORE_0.type
            deleter = EsDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.EVENTSTORE_0)
            edge_type = Pipeline.Mv.Edge.EVENTSTORE_0.type
            deleter = EsDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.EVENTSTORE_0)
            edge_type = Pipeline.Speed.Edge.EVENTSTORE_0.type
            deleter = EsDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        else:
            raise NotImplemented

    def run(self):
        helpers.silent_rm_r(self._edge_es_dirpath)
