import json
import os

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *

import edge.pipeline_helpers as helpers


def or_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    # speed
    if ui_entry_point in [Pipeline.Speed.Ui.HTTP_STREAMING_0]:
        used_ui_modules.update([Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0,
                                Pipeline.Speed.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.add(Pipeline.Speed.Edge.RENDERER_0)
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.RTSP_STREAMING_0]:
        used_ui_modules.update([Pipeline.Speed.Ui.RTSP_STREAMING_0, Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0,
                                Pipeline.Speed.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.add(Pipeline.Speed.Edge.RENDERER_1)
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.LANE_ENFORCEMENT_0]:
        used_ui_modules.update([Pipeline.Speed.Ui.RTSP_STREAMING_0, Pipeline.Speed.Ui.HTTP_STREAMING_0,
                                Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0, Pipeline.Speed.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.update([Pipeline.Speed.Edge.RENDERER_0, Pipeline.Speed.Edge.RENDERER_1])
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0]:
        used_ui_modules.update([Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.RTSP_STREAMING_0,
                                Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0, Pipeline.Speed.Ui.LANE_ENFORCEMENT_0])

        used_edge_modules.update([Pipeline.Speed.Edge.RENDERER_0, Pipeline.Speed.Edge.RENDERER_1])
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Speed.Ui.HTTP_STREAMING_0, Pipeline.Speed.Ui.RTSP_STREAMING_0,
                                Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0, Pipeline.Speed.Ui.LANE_ENFORCEMENT_0])

        creator_ctor = OrCreator.create

    # mv
    elif ui_entry_point in [Pipeline.Mv.Ui.HTTP_STREAMING_0]:
        used_ui_modules.update([Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Mv.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.add(Pipeline.Speed.Edge.RENDERER_0)
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.RTSP_STREAMING_0]:
        used_ui_modules.update([Pipeline.Mv.Ui.RTSP_STREAMING_0, Pipeline.Mv.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.add(Pipeline.Mv.Edge.RENDERER_1)
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.LANE_ENFORCEMENT_0]:
        used_ui_modules.update([Pipeline.Mv.Ui.RTSP_STREAMING_0, Pipeline.Mv.Ui.HTTP_STREAMING_0,
                                Pipeline.Mv.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.update([Pipeline.Mv.Edge.RENDERER_0, Pipeline.Mv.Edge.RENDERER_1])
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Mv.Ui.HTTP_STREAMING_0, Pipeline.Mv.Ui.RTSP_STREAMING_0,
                                Pipeline.Mv.Ui.LANE_ENFORCEMENT_0])
        creator_ctor = OrCreator.create

    # rtsp
    elif ui_entry_point in [Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.LANE_ENFORCEMENT_0]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.LANE_ENFORCEMENT_0])
        used_edge_modules.add(Pipeline.Rtsp.Edge.RENDERER_0)
        creator_ctor = OrCreator.create

    elif ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE]:
        used_ui_modules.update([Pipeline.Rtsp.Ui.HTTP_STREAMING_0, Pipeline.Rtsp.Ui.LANE_ENFORCEMENT_0])
        creator_ctor = OrCreator.create

    return creator_ctor


class OrCreator(BaseCreator):
    def __init__(self, id_main_setupstore_list):
        """
        :type id_main_setupstore_list: list of (str, dict, str)
        """
        BaseCreator.__init__(self)

        self._id_main_setupstore_list = id_main_setupstore_list

    @staticmethod
    def _create_lpr_rtsp(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: OrCreator | None
        """

        if ui_entry_point in [Pipeline.Rtsp.Ui.PIPELINE, Pipeline.Rtsp.Ui.HTTP_STREAMING_0,
                              Pipeline.Rtsp.Ui.LANE_ENFORCEMENT_0]:
            spool_dir = get_spool_dir(ui_pid)

            or0_ws_scene_dir = join_path(
                spool_dir, 'or%s-ws%s-scene' % (Pipeline.Rtsp.Edge.RENDERER_0.gid, Pipeline.Rtsp.Edge.WEBSERVER_0.gid))

            tr0_or0_scene_dir = join_path(
                spool_dir, 'tr%s-or%s-scene' % (Pipeline.Rtsp.Edge.TRACKER_0.gid, Pipeline.Rtsp.Edge.RENDERER_0.gid))

            or0_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.HTTP_STREAMING_0], None,
                ui_identity_to_jnode_map[Pipeline.Rtsp.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Rtsp.Edge.RENDERER_0, or0_ws_scene_dir, tr0_or0_scene_dir)

            return OrCreator([or0_args])
        else:
            return None

    @staticmethod
    def _create_speed(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: OrCreator | None
        """
        spool_dir = get_spool_dir(ui_pid)
        if ui_entry_point in [Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0, Pipeline.Speed.Ui.PIPELINE,
                              Pipeline.Speed.Ui.LANE_ENFORCEMENT_0]:
            or0_ws_scene_dir = join_path(
                spool_dir,
                'or%s-ws%s-scene/' % (Pipeline.Speed.Edge.RENDERER_0.gid, Pipeline.Speed.Edge.WEBSERVER_0.gid))

            tr0_or0_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Speed.Edge.TRACKER_0.gid, Pipeline.Speed.Edge.RENDERER_0.gid))

            or1_rs0_scene_dir = join_path(
                spool_dir,
                'or%s-rs%s-scene/' % (Pipeline.Speed.Edge.RENDERER_1.gid, Pipeline.Speed.Edge.RTSPSERVER_0.gid))

            tr0_or1_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Speed.Edge.TRACKER_0.gid, Pipeline.Speed.Edge.RENDERER_1.gid))

            or0_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.HTTP_STREAMING_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Speed.Edge.RENDERER_0, or0_ws_scene_dir, tr0_or0_scene_dir)

            or1_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.RTSP_STREAMING_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Speed.Edge.RENDERER_1, or1_rs0_scene_dir, tr0_or1_scene_dir)

            return OrCreator([or0_args, or1_args])

        elif ui_entry_point in [Pipeline.Speed.Ui.HTTP_STREAMING_0]:
            or0_ws_scene_dir = join_path(
                spool_dir,
                'or%s-ws%s-scene/' % (Pipeline.Speed.Edge.RENDERER_0.gid, Pipeline.Speed.Edge.WEBSERVER_0.gid))

            tr0_or0_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Speed.Edge.TRACKER_0.gid, Pipeline.Speed.Edge.RENDERER_0.gid))

            or0_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.HTTP_STREAMING_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Speed.Edge.RENDERER_0, or0_ws_scene_dir, tr0_or0_scene_dir)

            return OrCreator([or0_args])

        elif ui_entry_point in [Pipeline.Speed.Ui.RTSP_STREAMING_0]:
            or1_rs0_scene_dir = join_path(
                spool_dir,
                'or%s-rs%s-scene/' % (Pipeline.Speed.Edge.RENDERER_1.gid, Pipeline.Speed.Edge.RTSPSERVER_0.gid))

            tr0_or1_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Speed.Edge.TRACKER_0.gid, Pipeline.Speed.Edge.RENDERER_1.gid))

            or1_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.RTSP_STREAMING_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0],
                ui_identity_to_jnode_map[Pipeline.Speed.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Speed.Edge.RENDERER_1, or1_rs0_scene_dir, tr0_or1_scene_dir)

            return OrCreator([or1_args])

        else:
            raise NotImplemented

    @staticmethod
    def _create_mv(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: OrCreator | None
        """
        spool_dir = get_spool_dir(ui_pid)
        if ui_entry_point in [Pipeline.Mv.Ui.HTTP_STREAMING_0]:
            or0_ws_scene_dir = join_path(
                spool_dir,
                'or%s-ws%s-scene/' % (Pipeline.Mv.Edge.RENDERER_0.gid, Pipeline.Mv.Edge.WEBSERVER_0.gid))

            tr0_or0_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Mv.Edge.TRACKER_0.gid, Pipeline.Mv.Edge.RENDERER_0.gid))

            or0_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.HTTP_STREAMING_0], None,
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Mv.Edge.RENDERER_0, or0_ws_scene_dir, tr0_or0_scene_dir)

            return OrCreator([or0_args])

        elif ui_entry_point in [Pipeline.Mv.Ui.RTSP_STREAMING_0]:
            or1_rs0_scene_dir = join_path(
                spool_dir,
                'or%s-rs%s-scene/' % (Pipeline.Mv.Edge.RENDERER_1.gid, Pipeline.Mv.Edge.RTSPSERVER_0.gid))

            tr0_or1_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Mv.Edge.TRACKER_0.gid, Pipeline.Mv.Edge.RENDERER_1.gid))

            or1_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.RTSP_STREAMING_0], None,
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Mv.Edge.RENDERER_1, or1_rs0_scene_dir, tr0_or1_scene_dir)

            return OrCreator([or1_args])

        elif ui_entry_point in [Pipeline.Mv.Ui.LANE_ENFORCEMENT_0, Pipeline.Mv.Ui.PIPELINE]:
            or0_ws_scene_dir = join_path(
                spool_dir,
                'or%s-ws%s-scene/' % (Pipeline.Mv.Edge.RENDERER_0.gid, Pipeline.Mv.Edge.WEBSERVER_0.gid))

            tr0_or0_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Mv.Edge.TRACKER_0.gid, Pipeline.Mv.Edge.RENDERER_0.gid))

            or1_rs0_scene_dir = join_path(
                spool_dir,
                'or%s-rs%s-scene/' % (Pipeline.Mv.Edge.RENDERER_1.gid, Pipeline.Mv.Edge.RTSPSERVER_0.gid))

            tr0_or1_scene_dir = join_path(
                spool_dir,
                'tr%s-or%s-scene/' % (Pipeline.Mv.Edge.TRACKER_0.gid, Pipeline.Mv.Edge.RENDERER_1.gid))

            or0_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.HTTP_STREAMING_0], None,
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Mv.Edge.RENDERER_0, or0_ws_scene_dir, tr0_or0_scene_dir)

            or1_args = OrCreator._serialize(
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.RTSP_STREAMING_0], None,
                ui_identity_to_jnode_map[Pipeline.Mv.Ui.LANE_ENFORCEMENT_0],
                ui_pid, Pipeline.Mv.Edge.RENDERER_1, or1_rs0_scene_dir, tr0_or1_scene_dir)

            return OrCreator([or0_args, or1_args])

        else:
            raise NotImplemented

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: OrCreator
        """
        if ui_entry_point.ptype == PTYPE_SPEED:
            creator = OrCreator._create_speed(ui_pid, ui_entry_point, ui_identity_to_jnode_map)

        elif ui_entry_point.ptype == PTYPE_LPR_RTSP:
            creator = OrCreator._create_lpr_rtsp(ui_pid, ui_entry_point, ui_identity_to_jnode_map)

        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            creator = OrCreator._create_mv(ui_pid, ui_entry_point, ui_identity_to_jnode_map)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize(ui_streaming_jnode, ui_speed_enforcement_jnode, ui_lane_enforcement_jnode, ui_pid, edge_identity,
                   scene_dir, source_dir):
        """
        :type ui_streaming_jnode: dict
        :type ui_speed_enforcement_jnode: dict | None
        :type ui_lane_enforcement_jnode: dict
        :type ui_pid: str
        :type edge_identity: EdgeIdentity
        :type scene_dir: str
        :type source_dir: str
        :return: str, dict, str
        """
        codec = "JPEG"
        if edge_identity in [Pipeline.Mv.Edge.RENDERER_1, Pipeline.Speed.Edge.RENDERER_1]:
            codec = "RGBA"

        ui_live_jnode = ui_streaming_jnode['rw']['live']['videostream']
        ui_vehicle_jnode = ui_streaming_jnode['rw']['live']['renderer']['vehicle']

        main_conf_jnode = {
            "Base": "/etc/edge/setupstore/admin/actions/time.json",
            "Source": {
                "Url": source_dir
            },
            "Renderer": {
                "LrPlate": ui_vehicle_jnode['video-plate'],
                "LrDir": ui_vehicle_jnode['video-dir'],
                "LrLane": ui_vehicle_jnode['video-lane'],
                "LrPlateTrace": ui_vehicle_jnode['video-plate-trace'],
                "LrPlateUpscale": ui_vehicle_jnode['video-body-image'],

                "RaPosition": ui_vehicle_jnode.get('radar-position', 'never'),
                "RaSpeed": ui_vehicle_jnode.get('radar-speed', 'never'),
                "RaLength": ui_vehicle_jnode.get('radar-length', 'never'),
                "RaDir": ui_vehicle_jnode.get('radar-dir', 'never'),
                "ImageTime": ui_streaming_jnode['rw']['live']['renderer']['time'],
                "Encoder": {
                    "Codec": codec
                }
            },
            "image-store": {
                "format": codec,
                "max-width": ui_live_jnode['resolution']['max-width'],
                "max-height": ui_live_jnode['resolution']['max-height'],
                "endpoint": "/var/spool/edge/pipeline-%s/or%s-or%s-store/" % (ui_pid, edge_identity.gid, edge_identity.gid),
                "max-count": 8
            },
            "scene-event": {
                "max-file-count": 8,
                "endpoints": [scene_dir]
            }
        }
        if ui_speed_enforcement_jnode is not None and ui_speed_enforcement_jnode['rw']['enabled']:
            speed_limit = ui_speed_enforcement_jnode['rw']['speed-limit']
            main_conf_jnode['Renderer']['SpeedLimit'] = ui_speed_enforcement_jnode['rw']['speed-limit']
            main_conf_jnode['Renderer']['OverSpeedLimit'] = \
                speed_limit + ui_speed_enforcement_jnode['rw']['max-overspeed']

        if len(ui_lane_enforcement_jnode['rw']['road']['lanes']) > 0 and ui_vehicle_jnode['video-lane'] != 'never':
            main_conf_jnode['road-mask'] = {
                'lanes': ui_lane_enforcement_jnode['rw']['road']['lanes'],
                'resolution': {
                    'w': ui_lane_enforcement_jnode['rw']['road']['resolution']['w'],
                    'h': ui_lane_enforcement_jnode['rw']['road']['resolution']['h']
                }
            }

        edge_group = helpers.get_edge_group(ui_pid)
        edge_id = helpers.get_edge_id(edge_group, edge_identity)

        if ui_streaming_jnode['rw']['enabled']:
            setupstore_conf_text = get_setupstore_conf_text(edge_id)
        else:
            setupstore_conf_text = get_setupstore_conf_text(edge_id, active=False)

        return edge_id, main_conf_jnode, setupstore_conf_text

    def serialize(self):
        for (edge_id, main_conf_jnode, setup_store_text) in self._id_main_setupstore_list:
            edge_jnode_dirpath = get_edge_jnode_dirpath(EDGE_TYPE_RENDERER, edge_id)
            if not os.path.exists(edge_jnode_dirpath):
                helpers.silent_mkdir_r(edge_jnode_dirpath)

            main_conf_filepath = join_path(edge_jnode_dirpath, 'main.conf')
            with open(main_conf_filepath, 'w') as json_file:
                json.dump(main_conf_jnode, json_file, indent=2)

            setupstore_conf_filepath = join_path(edge_jnode_dirpath, 'setupstore.conf')
            with open(setupstore_conf_filepath, 'w') as text_file:
                text_file.write(setup_store_text)


class OrDeleter:
    def __init__(self, edge_rn_dirpath_list):
        """
        :type edge_rn_dirpath_list: list of str
        """
        self._edge_rn_dirpath_list = edge_rn_dirpath_list

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: OrDeleter
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            or0_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.RENDERER_0)
            or0_type = Pipeline.Rtsp.Edge.RENDERER_0.type
            deleter = OrDeleter([get_edge_jnode_dirpath(or0_type, or0_id)])
            return deleter

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            or0_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.RENDERER_0)
            or0_type = Pipeline.Mv.Edge.RENDERER_0.type

            or1_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.RENDERER_1)
            or1_type = Pipeline.Mv.Edge.RENDERER_1.type
            deleter = OrDeleter([get_edge_jnode_dirpath(or0_type, or0_id), get_edge_jnode_dirpath(or1_type, or1_id)])
            return deleter

        elif ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            or0_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.RENDERER_0)
            or0_type = Pipeline.Speed.Edge.RENDERER_0.type

            or1_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.RENDERER_1)
            or1_type = Pipeline.Speed.Edge.RENDERER_1.type
            deleter = OrDeleter([get_edge_jnode_dirpath(or0_type, or0_id), get_edge_jnode_dirpath(or1_type, or1_id)])
            return deleter

        else:
            raise NotImplemented

    def run(self):
        for edge_rn_dirpath in self._edge_rn_dirpath_list:
            helpers.silent_rm_r(edge_rn_dirpath)
