import json

from edge.path_helpers import *

from edge.pipeline_helpers import load_json_file, get_edge_group, get_edge_id


_RTSP_EDGE_MODULES = [
    Pipeline.Rtsp.Edge.LPR_0,
    Pipeline.Rtsp.Edge.EVENTSTORE_0,
    Pipeline.Rtsp.Edge.RENDERER_0,
    Pipeline.Rtsp.Edge.SPEEDCONTROL_0,
    Pipeline.Rtsp.Edge.TRACKER_0,
    Pipeline.Rtsp.Edge.RTSPGRABBER_0,
]

_MV_EDGE_MODULES = [
    Pipeline.Mv.Edge.LPR_0,
    Pipeline.Mv.Edge.GRAB_CONTROL_0,
    Pipeline.Mv.Edge.EVENTSTORE_0,
    Pipeline.Mv.Edge.RENDERER_0,
    Pipeline.Mv.Edge.RENDERER_1,
    Pipeline.Mv.Edge.SPEEDCONTROL_0,
    Pipeline.Mv.Edge.TRACKER_0,
    Pipeline.Mv.Edge.MEDIAGRABBER_0,
    Pipeline.Mv.Edge.RTSPSERVER_0,
    Pipeline.Mv.Edge.RAW_CODEC_0
]

_SPEED_EDGE_MODULES = [
    Pipeline.Speed.Edge.LPR_0,
    Pipeline.Speed.Edge.GRAB_CONTROL_0,
    Pipeline.Speed.Edge.EVENTSTORE_0,
    Pipeline.Speed.Edge.MEDIA_CODEC_0,
    Pipeline.Speed.Edge.RENDERER_0,
    Pipeline.Speed.Edge.RENDERER_1,
    Pipeline.Speed.Edge.SPEEDCONTROL_0,
    Pipeline.Speed.Edge.TRACKER_0,
    Pipeline.Speed.Edge.MEDIAGRABBER_0,
    Pipeline.Speed.Edge.RTSPSERVER_0,
    Pipeline.Speed.Edge.MEDIASTORE_0,
    Pipeline.Speed.Edge.RADAR_0,
    Pipeline.Speed.Edge.RAW_CODEC_0
]


def append_ss_group(ui_pid, ui_entry_point):
    """
    :type ui_pid: str
    :type ui_entry_point: UiIdentity
    """
    ss_jnode = load_json_file(SETUPSTORE_CONF_FILEPATH)
    edge_group = get_edge_group(ui_pid)

    if ui_entry_point.ptype == PTYPE_LPR_RTSP:
        spool_size = 512
        edge_modules = _RTSP_EDGE_MODULES

    elif ui_entry_point.ptype == PTYPE_LPR_MV:
        spool_size = 512
        edge_modules = _MV_EDGE_MODULES

    elif ui_entry_point.ptype == PTYPE_SPEED:
        spool_size = 1024
        edge_modules = _SPEED_EDGE_MODULES

    else:
        raise NotImplemented

    new_group_node = {
        "name": "pipeline-%s" % ui_pid,
        "spool-size": spool_size,
        "modules": map(lambda edge_identity:
                       get_edge_jnode_dirpath(edge_identity.type, get_edge_id(edge_group, edge_identity)),
                       edge_modules)
    }

    ss_jnode['groups'].append(new_group_node)

    with open(SETUPSTORE_CONF_FILEPATH, 'w') as json_file:
        json.dump(ss_jnode, json_file, indent=2)


def remove_ss_group(ui_pid):
    """
    :type ui_pid: str
    """
    ss_jnode = load_json_file(SETUPSTORE_CONF_FILEPATH)
    if not ss_jnode:
        raise RuntimeError

    groups_jnode = ss_jnode['groups']
    group_to_remove = filter(lambda group_jnode: group_jnode['name'] == 'pipeline-%s' % ui_pid, groups_jnode)
    if len(group_to_remove) == 1:
        groups_jnode.remove(group_to_remove[0])
        ss_jnode['groups'] = groups_jnode
        with open(SETUPSTORE_CONF_FILEPATH, 'w') as json_file:
            json.dump(ss_jnode, json_file, indent=2)

    elif len(group_to_remove) > 1:
        raise RuntimeError('fail: rm ss group')

