import jsonschema

import edge.eventlog as eventlog
from edge.admin.reply_helpers import *
from edge.admin.ui_types import *
from edge.path_helpers import *

from edge_creator import *
from _file_lock import *
import edge.pipeline_helpers as helpers
from ui_pipeline_set import get_pipeline_type


class UiLens:
    def __init__(self, handler, handler_args):
        """
        :type handler: object -> dict
        :type handler_args: object
        """

        self._handler = handler
        self._handler_args = handler_args

    @staticmethod
    def _handle_get(handler_args):
        """
        :type handler_args: (str, UiIdentity)
        """
        ui_pid, entry_point = handler_args

        videsource_status = helpers.get_videsource_status(ui_pid, entry_point.id)
        if videsource_status is None:
            eventlog.log_debug('fail: get videosource status')
            return reply_fail2(REPLY_SERVICE_UNAVAILABLE_503, error_code=FAIL_CODE_VIDEOSOURCE_ERROR,
                               error_text=FAIL_TEXT_VIDEOSOURCE_ERROR)
        elif videsource_status != 0:
            eventlog.log_debug('fail: videosource status error')
            return reply_fail2(REPLY_SERVICE_UNAVAILABLE_503, error_code=FAIL_CODE_VIDEOSOURCE_ERROR,
                               error_text=FAIL_TEXT_VIDEOSOURCE_ERROR)

        ui_jnode_filepath = get_ui_pipeline_jnode_filepath(ui_pid, entry_point.type, entry_point.id)
        old_jnode = helpers.load_json_file(ui_jnode_filepath)
        return reply_done(old_jnode['rw'])

    @staticmethod
    def _handle_put(handler_args):
        """
        :type handler_args: (str, UiIdentity, dict)
        :rtype: dict
        """
        ui_pid, entry_point, new_ui_jnode = handler_args

        videsource_status = helpers.get_videsource_status(ui_pid, entry_point.id)
        if videsource_status is None:
            eventlog.log_debug('fail: get videosource status')
            return reply_fail2(REPLY_SERVICE_UNAVAILABLE_503, error_code=FAIL_CODE_VIDEOSOURCE_ERROR,
                               error_text=FAIL_TEXT_VIDEOSOURCE_ERROR)
        elif videsource_status != 0:
            eventlog.log_debug('fail: videosource status error')
            return reply_fail2(REPLY_SERVICE_UNAVAILABLE_503, error_code=FAIL_CODE_VIDEOSOURCE_ERROR,
                               error_text=FAIL_TEXT_VIDEOSOURCE_ERROR)

        def _edit_ui_jnode_handler(ui_identity_to_jnode_map):
            """
            :type ui_identity_to_jnode_map: dict of UiIdentity: dict
            :rtype: bool
            """
            if ui_identity_to_jnode_map[entry_point]['rw'] == new_ui_jnode:
                return False

            # rotate
            new_iris = new_ui_jnode['iris']
            new_focus = new_ui_jnode['focus']

            old_iris = ui_identity_to_jnode_map[entry_point]['rw']['iris']
            old_focus = ui_identity_to_jnode_map[entry_point]['rw']['focus']

            # change jnode
            ui_identity_to_jnode_map[entry_point]['rw'] = new_ui_jnode

            if old_iris != new_iris:
                max_value_iris = ui_identity_to_jnode_map[entry_point]['ro']['max-iris']
                iris_value = int(new_iris - old_iris)
                if new_iris == 0:
                    os.system("python /etc/edge/psuctl.d/rotate_iris.py %s > /dev/null" % -max_value_iris)
                else:
                    os.system("python /etc/edge/psuctl.d/rotate_iris.py %s > /dev/null" % iris_value)

            if old_focus != new_focus:
                focus_value = int(new_focus - old_focus)
                max_value_focus = ui_identity_to_jnode_map[entry_point]['ro']['max-focus']
                if new_focus == 0:
                    os.system("python /etc/edge/psuctl.d/rotate_focus.py %s > /dev/null" % -max_value_focus)
                else:
                    os.system("python /etc/edge/psuctl.d/rotate_focus.py %s > /dev/null" % focus_value)

            return True

        try:
            generate(ui_pid, entry_point, _edit_ui_jnode_handler)
            return reply_done(new_ui_jnode)

        except LockError:
            return reply_fail(REPLY_CONFLICT_409)

    @staticmethod
    def _deserialize_get(query_args):
        """
        :type query_args: edge.admin.ui_types.UiQueryArgs
        :rtype: UiLens
        """
        try:
            ui_pid = query_args.get_path_info_item(UI_PIPELINE_PID_INDEX)
            ui_type = query_args.get_path_info_item(UI_PIPELINE_TYPE_INDEX)
            ui_id = query_args.get_path_info_item(UI_PIPELINE_ID_INDEX)
        except IndexError:
            return UiLens(handler=reply_fail, handler_args=REPLY_NOT_FOUND_404)

        ui_ptype = get_pipeline_type(ui_pid)
        if ui_ptype is None:
            return UiLens(handler=reply_fail, handler_args=REPLY_NOT_FOUND_404)

        entry_point = UiIdentity(ui_ptype, ui_type, ui_id)

        if query_args.query_string or query_args.raw_body:
            eventlog.log_debug('fail: bad-request')
            return UiLens(handler=reply_fail, handler_args=REPLY_BAD_REQUEST_400)

        return UiLens(UiLens._handle_get, (ui_pid, entry_point))

    @staticmethod
    def _deserialize_put(query_args):
        """
        :type query_args: edge.admin.ui_types.UiQueryArgs
        :rtype: UiLens
        """
        try:
            ui_pid = query_args.get_path_info_item(UI_PIPELINE_PID_INDEX)
            ui_type = query_args.get_path_info_item(UI_PIPELINE_TYPE_INDEX)
            ui_id = query_args.get_path_info_item(UI_PIPELINE_ID_INDEX)
        except IndexError:
            return UiLens(handler=reply_fail, handler_args=REPLY_NOT_FOUND_404)

        ui_ptype = get_pipeline_type(ui_pid)
        if ui_ptype is None:
            return UiLens(handler=reply_fail, handler_args=REPLY_NOT_FOUND_404)

        entry_point = UiIdentity(ui_ptype, ui_type, ui_id)

        if query_args.query_string:
            eventlog.log_debug('fail: bad-request')
            return UiLens(handler=reply_fail, handler_args=REPLY_BAD_REQUEST_400)

        # load new-ui-jnode
        try:
            new_ui_jnode = json.loads(query_args.raw_body) if query_args.raw_body else None
        except (IOError, ValueError):
            eventlog.log_debug('fail: load new-ui-jnode')
            return UiLens(handler=reply_fail, handler_args=REPLY_BAD_REQUEST_400)

        # load ui-schema-jnode
        try:
            ui_schema_filepath = get_ui_pipeline_schema_filepath(ui_pid, ui_type, ui_id)
            with open(ui_schema_filepath) as json_file:
                ui_schema_jnode = json.load(json_file)

        except (IOError, ValueError) as x:
            eventlog.log_debug("fail: load ui-schema-jnode (%s)" % x.message)
            return UiLens(handler=reply_fail, handler_args=REPLY_INTERNAL_SERVER_ERROR_500)

        # validate new-ui-jnode
        try:
            jsonschema.validate(new_ui_jnode, ui_schema_jnode)

        except (ValueError, jsonschema.ValidationError) as x:
            eventlog.log_debug("fail: validate new-ui-jnode (%s)" % x.message)
            return UiLens(handler=reply_fail, handler_args=REPLY_BAD_REQUEST_400)

        # all arguments are valid
        return UiLens(UiLens._handle_put, (ui_pid, entry_point, new_ui_jnode))

    @staticmethod
    def create(query_args):
        """
        :type query_args: edge.admin.ui_types.UiQueryArgs
        :rtype: UiLens
        """
        if query_args.http_method == 'GET':
            return UiLens._deserialize_get(query_args)

        elif query_args.http_method == 'PUT':
            return UiLens._deserialize_put(query_args)

        else:
            eventlog.log_debug("fail: method-not-allowed")
            return UiLens(handler=reply_fail, handler_args=REPLY_METHOD_NOT_ALLOWED_405)

    def handle(self):
        """
        :rtype: dict
        """
        return self._handler(self._handler_args)
