from edge.admin.reply_helpers import *
from edge.fs_helpers import create_schema_symlink, create_jnode_from_default

import ui_pipeline_set
from edge_creator import *
from _edge_setupstore import append_ss_group
import edge.pipeline_helpers as helpers
from ui_pipeline import delete_pipeline, get_pipeline_ext_jnode

_TIME_DEF_JNODE_FILEPATH = '/opt/edge/share/setupstore/ui/time-jnode.json'

_UI_IDENTITY_TO_DATASOURCE_MAP = [
    (Pipeline.Mv.Ui.VIDEOSOURCE_0, True),
    (Pipeline.Mv.Ui.LENS_0, True),
    (Pipeline.Mv.Ui.GEOLOCATION_0, False),
    (Pipeline.Mv.Ui.LPR_0, False),
    (Pipeline.Mv.Ui.LANE_ENFORCEMENT_0, False),
    (Pipeline.Mv.Ui.EVENTSTORE_0, False),
    (Pipeline.Mv.Ui.HTTP_STREAMING_0, False),
    (Pipeline.Mv.Ui.RTSP_STREAMING_0, False),
    (Pipeline.Mv.Ui.FACT_RULES_0, False)
]


def _create_ui_jnode_set(ui_pid, raw_body):
    """
    :type ui_pid: str
    :type raw_body: str
    """
    args = json.loads(raw_body)
    serial = args['serial']

    # read def_jnode
    ui_identity_to_def_jnode_map = dict()
    for (ui_identity, _) in _UI_IDENTITY_TO_DATASOURCE_MAP:
        def_ui_jnode_filepath = get_ui_def_jnode_filepath(ui_identity, raw_body)
        def_ui_jnode = helpers.load_json_file(def_ui_jnode_filepath)
        ui_identity_to_def_jnode_map[ui_identity] = def_ui_jnode

    # edit
    ui_identity_to_def_jnode_map[Pipeline.Mv.Ui.EVENTSTORE_0]['rw']['channel'] = int(ui_pid)
    ui_identity_to_def_jnode_map[Pipeline.Mv.Ui.VIDEOSOURCE_0]['rw']['serial'] = str(serial)

    eventlog.log_debug(json.dumps(ui_identity_to_def_jnode_map[Pipeline.Mv.Ui.VIDEOSOURCE_0]))

    # write
    for ui_identity, ui_jnode in ui_identity_to_def_jnode_map.iteritems():
        ui_jnode_dirpath = get_ui_pipeline_jnode_dirpath(ui_pid, ui_identity)
        if not os.path.exists(ui_jnode_dirpath):
            helpers.silent_mkdir_r(ui_jnode_dirpath)

        ui_jnode_filepath = get_ui_pipeline_jnode_filepath(ui_pid, ui_identity.type, ui_identity.id)
        with open(ui_jnode_filepath, 'w') as json_file:
            json.dump(ui_jnode, json_file, indent=2)

        ui_schema_filepath = get_ui_pipeline_schema_filepath(ui_pid, ui_identity.type, ui_identity.id)
        create_schema_symlink(ui_schema_filepath, ui_jnode_filepath)


def create_mv_pipeline(body, ui_pid):
    """
    :type body: str
    :type ui_pid: str
    """
    entry_point = Pipeline.Mv.Ui.PIPELINE
    pipeline_jnode_set = helpers.get_pipeline_jnode_set2()
    if pipeline_jnode_set is None:
        eventlog.log_debug("fail: internal-server-error")
        return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)

    # noinspection PyBroadException
    try:
        # create ui-jnode-set
        _create_ui_jnode_set(ui_pid, body)
        # create edge-jnode-set
        generate(ui_pid, entry_point, None)

        # update pipeline.json
        visible_ui_type_set = map(lambda (module, datasource): (module.type, datasource),
                                  _UI_IDENTITY_TO_DATASOURCE_MAP)

        pipeline_jnode = ui_pipeline_set.add_pipeline(pipeline_jnode_set, ui_pid, entry_point.ptype, visible_ui_type_set)
        pipeline_ext_jnode = get_pipeline_ext_jnode(pipeline_jnode)

        # create time-jnode
        create_jnode_from_default(_TIME_DEF_JNODE_FILEPATH, UI_TIME_JNODE_FILEPATH)

        # update setupstore config
        append_ss_group(ui_pid, entry_point)
        # restart webserver with PCD
        helpers.restart_webserver_lazy()

        helpers.sighup_setupstore()

        return reply_done(pipeline_ext_jnode)

    except NotImplementedError:
        eventlog.log_debug('fail: camera not supported')
        return reply_fail(REPLY_BAD_REQUEST_400)

    except LockError:
        return reply_fail(REPLY_CONFLICT_409)

    except:
        eventlog.log_debug_last_exc()

    delete_pipeline(ui_pid)
    return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)
