from edge.admin.reply_helpers import *
from edge.fs_helpers import create_schema_symlink, create_jnode_from_default

import ui_pipeline_set
from edge_creator import *
from _edge_setupstore import append_ss_group
import edge.pipeline_helpers as helpers
from ui_pipeline import delete_pipeline, get_pipeline_ext_jnode

_TIME_DEF_JNODE_FILEPATH = '/opt/edge/share/setupstore/ui/time-jnode.json'

_UI_IDENTITY_TO_DATASOURCE_MAP = [
    (Pipeline.Rtsp.Ui.GEOLOCATION_0, False),
    (Pipeline.Rtsp.Ui.VIDEOSOURCE_0, True),
    (Pipeline.Rtsp.Ui.LPR_0, False),
    (Pipeline.Rtsp.Ui.LANE_ENFORCEMENT_0, False),
    (Pipeline.Rtsp.Ui.EVENTSTORE_0, False),
    (Pipeline.Rtsp.Ui.HTTP_STREAMING_0, False),
    (Pipeline.Rtsp.Ui.FACT_RULES_0, False)
]


class CreatorError(RuntimeError):
    pass


def _create_ui_jnode_set(ui_pid):
    """
    :type ui_pid: str
    """

    # read def_jnode
    ui_identity_to_def_jnode_map = dict()
    for (ui_identity, _) in _UI_IDENTITY_TO_DATASOURCE_MAP:
        def_ui_jnode_filepath = get_ui_def_jnode_filepath(ui_identity, None)
        def_ui_jnode = helpers.load_json_file(def_ui_jnode_filepath)
        ui_identity_to_def_jnode_map[ui_identity] = def_ui_jnode

    # edit
    ui_identity_to_def_jnode_map[Pipeline.Rtsp.Ui.EVENTSTORE_0]['rw']['channel'] = int(ui_pid)

    # write
    for ui_identity, ui_jnode in ui_identity_to_def_jnode_map.iteritems():
        ui_jnode_dirpath = get_ui_pipeline_jnode_dirpath(ui_pid, ui_identity)
        if not os.path.exists(ui_jnode_dirpath):
            helpers.silent_mkdir_r(ui_jnode_dirpath)

        ui_jnode_filepath = get_ui_pipeline_jnode_filepath(ui_pid, ui_identity.type, ui_identity.id)
        with open(ui_jnode_filepath, 'w') as json_file:
            json.dump(ui_jnode, json_file, indent=2)

        ui_schema_filepath = get_ui_pipeline_schema_filepath(ui_pid, ui_identity.type, ui_identity.id)
        create_schema_symlink(ui_schema_filepath, ui_jnode_filepath)


def _generator(ui_pid):
    """
    :type ui_pid: str
    """
    entry_point = Pipeline.Rtsp.Ui.PIPELINE

    try:
        generate(ui_pid, entry_point, None)
        return REPLY_SUCCESS_200

    except LockError:
        return REPLY_CONFLICT_409


def create_rtsp_pipeline(ui_pid):
    """
    :type ui_pid: str
    :rtype: dict
    """
    reply_code = REPLY_INTERNAL_SERVER_ERROR_500

    entry_point = Pipeline.Rtsp.Ui.PIPELINE
    pipeline_jnode_set = helpers.get_pipeline_jnode_set2()
    if pipeline_jnode_set is None:
        eventlog.log_debug("fail: internal-server-error")
        return reply_fail(reply_code)

    # noinspection PyBroadException
    try:
        # create ui-jnode-set
        _create_ui_jnode_set(ui_pid)

        # create edge-jnode-set
        reply_code = _generator(ui_pid)
        if reply_code != REPLY_SUCCESS_200:
            raise CreatorError

        # update pipeline.json
        visible_ui_type_set = map(lambda (ui_module, datasource): (ui_module.type, datasource),
                                  _UI_IDENTITY_TO_DATASOURCE_MAP)

        pipeline_jnode = ui_pipeline_set.add_pipeline(pipeline_jnode_set, ui_pid, PTYPE_LPR_RTSP, visible_ui_type_set)
        pipeline_ext_jnode = get_pipeline_ext_jnode(pipeline_jnode)

        # create time-jnode
        create_jnode_from_default(_TIME_DEF_JNODE_FILEPATH, UI_TIME_JNODE_FILEPATH)

        # update setupstore config
        append_ss_group(ui_pid, entry_point)

        # restart webserver with PCD
        helpers.restart_webserver_lazy()

        helpers.sighup_setupstore()

        return reply_done(pipeline_ext_jnode)

    except CreatorError:
        pass

    except:
        eventlog.log_debug_last_exc()

    delete_pipeline(ui_pid)
    return reply_fail(reply_code)
