from edge.admin.reply_helpers import *
from edge.fs_helpers import create_schema_symlink, create_jnode_from_default

import ui_pipeline_set
from edge_creator import *
from _edge_setupstore import append_ss_group
import edge.pipeline_helpers as helpers
import edge.db as db_helpers
from ui_pipeline import delete_pipeline, get_pipeline_ext_jnode

_TIME_DEF_JNODE_FILEPATH = '/opt/edge/share/setupstore/ui/time-jnode.json'

_UI_IDENTITY_TO_DATASOURCE_MAP = [
    (Pipeline.Speed.Ui.VIDEOSOURCE_0, True),
    (Pipeline.Speed.Ui.LENS_0, True),
    (Pipeline.Speed.Ui.GEOLOCATION_0, False),
    (Pipeline.Speed.Ui.LPR_0, False),
    (Pipeline.Speed.Ui.LANE_ENFORCEMENT_0, False),
    (Pipeline.Speed.Ui.EVENTSTORE_0, False),
    (Pipeline.Speed.Ui.MEDIASTORE_0, False),
    (Pipeline.Speed.Ui.HTTP_STREAMING_0, False),
    (Pipeline.Speed.Ui.RTSP_STREAMING_0, False),
    (Pipeline.Speed.Ui.LOCATION_0, False),
    (Pipeline.Speed.Ui.SPEED_ENFORCEMENT_0, False),
    (Pipeline.Speed.Ui.FACT_RULES_0, False)
]


def _create_ui_jnode_set(ui_pid, raw_body):
    """
    :type ui_pid: str
    :type raw_body: str
    """
    args = json.loads(raw_body)
    serial = args['serial']

    # read def_jnode
    ui_identity_to_def_jnode_map = dict()
    for (ui_identity, _) in _UI_IDENTITY_TO_DATASOURCE_MAP:
        def_ui_jnode_filepath = get_ui_def_jnode_filepath(ui_identity, raw_body)
        def_ui_jnode = helpers.load_json_file(def_ui_jnode_filepath)
        ui_identity_to_def_jnode_map[ui_identity] = def_ui_jnode

    # edit
    ui_identity_to_def_jnode_map[Pipeline.Speed.Ui.EVENTSTORE_0]['rw']['channel'] = int(ui_pid)
    ui_identity_to_def_jnode_map[Pipeline.Speed.Ui.VIDEOSOURCE_0]['rw']['serial'] = str(serial)

    # write
    for ui_identity, ui_jnode in ui_identity_to_def_jnode_map.iteritems():
        ui_jnode_dirpath = get_ui_pipeline_jnode_dirpath(ui_pid, ui_identity)
        if not os.path.exists(ui_jnode_dirpath):
            helpers.silent_mkdir_r(ui_jnode_dirpath)

        ui_jnode_filepath = get_ui_pipeline_jnode_filepath(ui_pid, ui_identity.type, ui_identity.id)
        with open(ui_jnode_filepath, 'w') as json_file:
            json.dump(ui_jnode, json_file, indent=2)

        ui_schema_filepath = get_ui_pipeline_schema_filepath(ui_pid, ui_identity.type, ui_identity.id)
        eventlog.log_debug(ui_schema_filepath)
        create_schema_symlink(ui_schema_filepath, ui_jnode_filepath)


def _get_new_pid(pipeline_jnode_set):
    """
    :type pipeline_jnode_set: list of dict
    :rtype: str | None
    """
    used_ui_pid_set = set(map(lambda pipeline_jnode: int(pipeline_jnode['id']), pipeline_jnode_set))

    ui_pid = None
    for ui_pid_candidate in xrange(0, 32):
        if ui_pid_candidate not in used_ui_pid_set:
            ui_pid = str(ui_pid_candidate)
            eventlog.log_debug("new-pipeline-id: %s" % ui_pid)
            break
    return ui_pid


def create_speed_pipeline(vendor, model, serial):
    """
    :type vendor: str
    :type model: str
    :type serial: str
    """
    # in this case mb better to get 3 stings and  create arg for "_create_ui_jnode_set": ??
    # vendor = ''
    # model = ''
    # serial = ''
    jnode = {
        "vendor": vendor,
        "model": model,
        "serial": serial
    }

    entry_point = Pipeline.Speed.Ui.PIPELINE
    pipeline_jnode_set = helpers.get_pipeline_jnode_set2()
    if pipeline_jnode_set is None:
        eventlog.log_debug("fail: internal-server-error")
        return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)

    ui_pid = _get_new_pid(pipeline_jnode_set)
    if ui_pid is None:
        eventlog.log_debug("fail: service-unavailable")
        return reply_fail(REPLY_SERVICE_UNAVAILABLE_503)

    def _handler(db_link):
        with db_link:
            db_helpers.edit_post(db_link, post_args)
            db_helpers.edit_channel(db_link, post_args, channel_args)

    # noinspection PyBroadException
    try:
        # create db_post
        db_config_jnode = helpers.load_db_json()
        db_args = db_helpers.DbArgs.deserialize(db_config_jnode['db'])
        post_args = db_helpers.DbPostArgs.deserialize(db_config_jnode['post'])
        channel_args = db_helpers.DbChannelArgs(ident=int(ui_pid))
        db_helpers.with_db(db_args, _handler)

        # create ui-jnode-set
        _create_ui_jnode_set(ui_pid, json.dumps(jnode))

        # create edge-jnode-set
        generate(ui_pid, entry_point, None)

        # update pipeline.json
        visible_ui_type_set = map(lambda (module, datasource): (module.type, datasource),
                                  _UI_IDENTITY_TO_DATASOURCE_MAP)

        pipeline_jnode = ui_pipeline_set.add_pipeline(pipeline_jnode_set, ui_pid, entry_point.ptype,
                                                      visible_ui_type_set, internal=True)
        pipeline_ext_jnode = get_pipeline_ext_jnode(pipeline_jnode)

        # create time-jnode
        create_jnode_from_default(_TIME_DEF_JNODE_FILEPATH, UI_TIME_JNODE_FILEPATH)

        # update setupstore config
        append_ss_group(ui_pid, entry_point)
        # restart webserver with PCD
        helpers.restart_webserver_lazy()

        helpers.sighup_setupstore()

        return reply_done(pipeline_ext_jnode)

    except NotImplementedError:
        return 'fail: camera not supported'

    except db_helpers.DbError:
        pass

    except LockError:
        return reply_fail(REPLY_CONFLICT_409)

    except:
        eventlog.log_debug_last_exc()

    delete_pipeline(ui_pid)
    eventlog.log_debug('fail: internal-server-error')
    return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)
