import json
import os

import edge.eventlog as eventlog
import edge.pipeline_helpers as pipeline_helpers
# from edge.fs_helpers import create_jnode_from_default

# _PIPELINE_SET_FILEPATH = '/etc/edge/setupstore/admin/actions/pipeline.json'
# _PIPELINE_DEFSET_FILEPATH = '/opt/edge/share/setupstore/ui/pipeline-jnode.json'


def validate_ui_pid(ui_pid):
    """
    :type ui_pid: str
    :rtype: bool
    """
    if not ui_pid:
        return False

    with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'r') as json_file:
        pipelines_jnode = json.load(json_file)
    if not pipelines_jnode:
        return False

    pipeline_jnode_set = filter(lambda item_jnode: item_jnode['id'] == ui_pid, pipelines_jnode)
    if len(pipeline_jnode_set) != 1:
        return False

    return True


def validate_ui_type(ui_pid, ui_type):
    """
    :type ui_pid: str
    :type ui_type: str
    :rtype: bool
    """
    if not (ui_pid and ui_type):
        return False

    with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'r') as json_file:
        pipelines_jnode = json.load(json_file)
    if not pipelines_jnode:
        return False

    pipeline_jnode_set = filter(lambda item_jnode: item_jnode['id'] == ui_pid, pipelines_jnode)
    if len(pipeline_jnode_set) != 1:
        return False

    # noinspection PyTypeChecker
    modules_jnode = pipeline_jnode_set[0]['modules']
    module_jnode_set = filter(lambda item_jnode: item_jnode['type'] == ui_type, modules_jnode)
    if len(module_jnode_set) != 1:
        return False

    return True


def validate_ui_id(ui_pid, ui_type, ui_id):
    """
    :type ui_pid: str
    :type ui_type: str
    :type ui_id: str
    :rtype: bool
    """
    if not (ui_pid and ui_type and ui_id):
        return False

    with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'r') as json_file:
        pipelines_jnode = json.load(json_file)
    if not pipelines_jnode:
        return False

    pipeline_jnode_set = filter(lambda item_jnode: item_jnode['id'] == ui_pid, pipelines_jnode)
    if len(pipeline_jnode_set) != 1:
        return False

    # noinspection PyTypeChecker
    modules_jnode = pipeline_jnode_set[0]['modules']
    module_jnode_set = filter(lambda item_jnode: item_jnode['type'] == ui_type and item_jnode['id'] == ui_id,
                              modules_jnode)
    if len(module_jnode_set) != 1:
        return False

    return True


def get_pipeline_type(ui_pid):
    """
    :type ui_pid: str
    :rtype: str | None
    """
    pipeline_jnode_set = pipeline_helpers.get_pipeline_jnode_set2()

    pipeline_jnode = filter(lambda item_jnode: item_jnode['id'] == ui_pid, pipeline_jnode_set)
    if len(pipeline_jnode) != 1:
        eventlog.log_debug('fail: wrong pid')
        return None

    return pipeline_jnode[0]['type']


def add_pipeline(pipeline_set_jnode, ui_pid, ui_ptype, ui_type_set, internal=False):
    """
    :type pipeline_set_jnode: list of dict
    :type ui_pid: str
    :type ui_ptype: str
    :type ui_type_set: list of (str, bool)
    :type internal: bool
    :rtype: dict
    """
    ui_modules_jnode = []
    for (ui_type, source) in ui_type_set:
        ui_modules_jnode.append(
            {
                "datasource": source,
                "id": "0",
                "type": "%s" % ui_type,
                "title": "pipeline.%s" % ui_type
            })

    pipeline_jnode = {
        "id": "%s" % ui_pid,
        "internal": internal,
        "type": ui_ptype,
        "modules": ui_modules_jnode
    }

    pipeline_set_jnode.append(pipeline_jnode)

    with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'w') as json_file:
        json.dump(pipeline_set_jnode, json_file, indent=2)

    return pipeline_jnode


def remove_pipeline(ui_pid):
    """
    :type ui_pid: str
    :rtype: dict
    """
    with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'r') as json_file:
        pipeline_jnode_set = json.load(json_file)

    new_pipeline_set_jnode = filter(lambda pipeline_jnode: pipeline_jnode['id'] != ui_pid, pipeline_jnode_set)

    with open(pipeline_helpers.PIPELINE_SET_FILEPATH, 'w') as json_file:
        json.dump(new_pipeline_set_jnode, json_file, indent=2)



