#!/usr/bin/python
import sys
import json

from os.path import join as join_path

import edge.eventlog as eventlog
from edge.admin.reply_helpers import *
import edge.admin.ui_types as ui_types


BASE_NLS_DIR = '/opt/edge/share/setupstore/nls'


def _run(query_args):
    """
    :type query_args: ui_types.UiQueryArgs
    :rtype: dict
    """
    if not query_args.path_info:
        eventlog.log_debug('fail: empty path-info')
        return reply_fail(REPLY_BAD_REQUEST_400)

    # Dispatch http-method.
    if query_args.http_method == 'GET':
        nls_file_path = join_path(BASE_NLS_DIR, *query_args.path_info)
        try:
            eventlog.log_debug('loading nls %s' % nls_file_path)
            with open(nls_file_path, 'r') as nls_file:
                nls_jnode = json.load(nls_file)
            return reply_done(nls_jnode)

        except Exception as x:
            eventlog.log_debug("fail: load nls (%s)" % x.message)
            return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)

    else:
        eventlog.log_debug('fail: method (%s) not allowed' % query_args.http_method)
        return reply_fail(REPLY_METHOD_NOT_ALLOWED_405)


def _main():
    eventlog.open_log('cgi-nls', 'debug')
    exit_code = 1
    # noinspection PyBroadException
    try:
        query_args = ui_types.UiQueryArgs.deserialize()

        response_dict = _run(query_args)
        print json.dumps(dict(response_dict), separators=(',', ':'))
        exit_code = 0

    except:
        eventlog.log_debug_last_exc()
    finally:
        eventlog.close_log()

    sys.exit(exit_code)


if __name__ == '__main__':
    _main()
