#!/usr/bin/python
import json
import os
import sys
from os.path import join as join_path

import edge.admin.ui_types as ui_types
import edge.eventlog as eventlog
from edge.admin.reply_helpers import *
from edge.fs_helpers import create_schema_symlink

BASE_UI_SCHEMA_DIR = '/etc/edge/setupstore/schemas'
BASE_UI_JNODE_DIR = '/etc/edge/setupstore/admin/actions'


def _run(query_args):
    """
    :type query_args: ui_types.UiQueryArgs
    :rtype: dict
    """
    # Dispatch http-method.
    if query_args.http_method == 'GET':
        schema_filepath = join_path(BASE_UI_SCHEMA_DIR, *query_args.path_info)
        if not os.path.islink(schema_filepath) or not os.path.exists(os.readlink(schema_filepath)):
            ui_jnode_filepath = join_path(BASE_UI_JNODE_DIR, *query_args.path_info)
            if not os.path.exists(ui_jnode_filepath):
                eventlog.log_debug('fail: %s not found' % ui_jnode_filepath)
                return reply_fail(REPLY_NOT_FOUND_404)

            create_schema_symlink(schema_filepath, ui_jnode_filepath)

        try:
            eventlog.log_debug('schema_filepath: %s' % schema_filepath)
            with open(schema_filepath, 'r') as json_file:
                schema_jnode = json.load(json_file)
                if not schema_jnode:
                    eventlog.log_debug("fail: load def-schema")

        except (IOError, ValueError) as x:
            eventlog.log_debug("fail: (%s)" % x.message)
            return reply_fail(REPLY_INTERNAL_SERVER_ERROR_500)

        response_jnode = reply_done(schema_jnode)

    else:
        eventlog.log_debug('fail: method (%s) not allowed' % query_args.http_method)
        response_jnode = reply_fail(REPLY_METHOD_NOT_ALLOWED_405)

    return response_jnode


def _main():
    eventlog.open_log('cgi-httpget', 'debug')
    exit_code = 1
    # noinspection PyBroadException
    try:
        query_args = ui_types.UiQueryArgs.deserialize()

        response_dict = _run(query_args)
        print json.dumps(dict(response_dict), separators=(',', ':'))
        exit_code = 0
    except:
        eventlog.log_debug_last_exc()
    finally:
        eventlog.close_log()

    sys.exit(exit_code)


if __name__ == '__main__':
    _main()
