import time
import datetime
from os.path import join as join_path, exists
from ujson import dump as ujson_dump
import edge.eventlog as eventlog
from edge.fs_helpers import silent_mkdir_r

from _args import PlateAbsentDbArgs
import _models


class IPlateAbsentDb:
    def __init__(self):
        pass

    def handle_fact(self, fact, fact_images):
        """
        :type fact: _models.Fact
        :type fact_images: _models.FactImages
        """
        raise NotImplemented


class StubPlateAbsentDb(IPlateAbsentDb):
    def __init__(self):
        IPlateAbsentDb.__init__(self)

    def handle_fact(self, *args, **kwargs):
        pass


class PlateAbsentDb(IPlateAbsentDb):
    def __init__(self, args):
        """
        :type args: PlateAbsentDbArgs
        """
        IPlateAbsentDb.__init__(self)
        self._root_dirpath = args.db_directory
        silent_mkdir_r(self._root_dirpath)

        self._lanes = args.lanes
        self._min_time = args.min_time
        self._max_time = args.max_time

    def handle_fact(self, fact, fact_images):
        """
        :type fact: _models.Fact
        :type fact_images: _models.FactImages
        """
        image_struct_time = time.gmtime(fact_images.ts / 1000000)
        image_time = datetime.time(image_struct_time.tm_hour, image_struct_time.tm_min)

        lane_valid = True
        if len(self._lanes) and fact.lane not in self._lanes:
            lane_valid = False

        if lane_valid and self._min_time < image_time < self._max_time:
            image_filepath = join_path(self._root_dirpath, str(fact_images.ts))

            fact_jnode = _models.Fact.serialize(fact, fact_images.ts, images_jnode=[{
                'name': 'scene',
                'blob': image_filepath
            }])

            try:
                if not exists(self._root_dirpath):
                    silent_mkdir_r(self._root_dirpath)

                with open(image_filepath, 'wb') as jpeg_file:
                    jpeg_file.write(fact_images.scene_blob)

                with open(image_filepath + '-scene', 'w') as json_file:
                    ujson_dump(fact_jnode, json_file, ensure_ascii=False)

            except IOError:
                eventlog.log_debug('fail: save event on disk')
