import os
import signal
import sys
import ujson
from time import sleep
from twisted.internet import reactor

from edge.argset import ArgSet
import edge.eventlog as eventlog

from __version import VERSION_STRING
from _args import KernelArgs
from _kernel import Kernel


# noinspection PyUnusedLocal
def handle_sighup(signal_nubmer, interrupted_stack_frame):
    # traceback.print_stack(interrupted_stack_frame)
    pass


# noinspection PyUnresolvedReferences,PyBroadException
def run_module(raw_arg_set):
    """
    :type raw_arg_set: list of str
    """
    arg_set = ArgSet(raw_arg_set)
    if arg_set.mode == ArgSet.MODE_NONE:
        print "The command line's arguments are invalid"
        sys.exit(1)

    if arg_set.mode == ArgSet.MODE_VERSION:
        print VERSION_STRING
        return

    if arg_set.mode == ArgSet.MODE_STARTUP:
        module_conf_name = 'speedcontrol.{}'.format(arg_set.module_id)
        eventlog.open_log(module_conf_name, arg_set.log_level)
        eventlog.log_info('startup...')

        reactor.addSystemEventTrigger('after', 'startup', eventlog.log_info, 'startup... done')
        reactor.addSystemEventTrigger('before', 'shutdown', eventlog.log_info, 'shutdown...')
        signal.signal(signal.SIGHUP, handle_sighup)

        main_conf_abspath = os.path.join('/etc/edge', module_conf_name, 'main.conf')
        try:
            kernel_args = None
            trottle_wait_args = False
            while kernel_args is None:
                if trottle_wait_args:
                    sleep(15)

                trottle_wait_args = True

                # deserialize main.conf
                try:
                    with open(main_conf_abspath) as main_conf_jnode:
                        main_conf_jnode = ujson.load(main_conf_jnode)

                except Exception as x:
                    eventlog.log_error('failed to load %s (%s)' % (main_conf_abspath, x))
                    continue

                # deserialize kernerl-args
                try:
                    kernel_args = KernelArgs.deserialize(main_conf_jnode, arg_set.log_level)

                except Exception as x:
                    eventlog.log_error('failed to read args (%s)' % x)
                    continue

            kernel = Kernel(reactor, kernel_args)
            kernel.run()

        finally:
            eventlog.log_info('shutdown... done')
            eventlog.close_log()

        return

if __name__ == '__main__':
    run_module(sys.argv)
