import collections


class LrRaOidMatchToutStore:
    def __init__(self, twisted_reactor, lr_ra_oid_match_store_tout):
        self._twisted_reactor = twisted_reactor
        self._lr_ra_oid_match_store_tout_sec = lr_ra_oid_match_store_tout

        self._lr_oid_to_delayed_call_map = dict()
        self._lr_oid_to_ra_oid_to_match_count_map = collections.defaultdict(collections.Counter)

        self._lr_last_source_id = None

    def _handle_lr_oid_tout(self, lr_oid):
        self._lr_oid_to_ra_oid_to_match_count_map.pop(lr_oid, None)
        self._lr_oid_to_delayed_call_map.pop(lr_oid, None)

    def add(self, lr_source_id, tr_scene_tr_items):
        """
        :type lr_source_id: list of int
        :type tr_scene_tr_items: list of _models.TrSceneItem
        """

        if self._lr_last_source_id == lr_source_id:
            for tr_scene_item in tr_scene_tr_items:
                # new lr_oid will be inserted.
                lr_oid = tr_scene_item.lr_scene_item.oid
                rm_delayed_call = self._lr_oid_to_delayed_call_map.get(lr_oid, None)
                if rm_delayed_call:
                    rm_delayed_call.reset(self._lr_ra_oid_match_store_tout_sec)

                else:

                    rm_delayed_call = self._twisted_reactor.callLater(
                        self._lr_ra_oid_match_store_tout_sec, self._handle_lr_oid_tout, lr_oid)
                    self._lr_oid_to_delayed_call_map[lr_oid] = rm_delayed_call

                self._lr_oid_to_ra_oid_to_match_count_map[lr_oid][tr_scene_item.ra_scene_item.oid] += 1

        else:

            self._lr_oid_to_ra_oid_to_match_count_map.clear()
            self._lr_oid_to_delayed_call_map.clear()
            self._lr_last_source_id = lr_source_id

    def try_get_most_mached_ra_track(self, lr_source_id, lr_track_oid_list, get_ra_track_handler):
        """
        :type lr_source_id: list of int
        :type lr_track_oid_list: list of int
        :type get_ra_track_handler: (object, int) -> (_models.RaTrack | None)
        :rtype: _models.RaTrack | None
        """
        if self._lr_last_source_id != lr_source_id:
            return None

        final_ra_oid_to_match_count_map = collections.Counter()

        for lr_oid in lr_track_oid_list:
            ra_oid_to_match_count_map = self._lr_oid_to_ra_oid_to_match_count_map.get(lr_oid, None)

            if ra_oid_to_match_count_map:
                final_ra_oid_to_match_count_map += ra_oid_to_match_count_map

        # sort entries by count in desc order
        ra_oid_desc_list = sorted(final_ra_oid_to_match_count_map,
                                  key=final_ra_oid_to_match_count_map.get, reverse=True)

        for ra_oid in ra_oid_desc_list:
            ra_track = get_ra_track_handler(ra_oid)
            if ra_track:
                return ra_track

        return None
