import _models


class _RaTrackContext:
    def __init__(self, ra_track, rm_delayed_call):
        """
        :type ra_track: _models.RaTrack
        :type rm_delayed_call: twisted.internet.interfaces.IDelayedCall
        """
        self.ra_track = ra_track
        self.rm_delayed_call = rm_delayed_call
        self.rm_request = False


class RaTrackToutStore:
    def __init__(self, twisted_reactor, ra_track_store_tout):
        """
        :type twisted_reactor: twisted.internet.interfaces.IReactorTime
        :type ra_track_store_tout: int
        """
        self._twisted_reactor = twisted_reactor
        self._ra_track_store_tout_sec = ra_track_store_tout
        self._ra_oid_to_context_map = {}

        self._ra_track_tout_handler = None

    def set_ra_track_tout_handler(self, handler):
        self._ra_track_tout_handler = handler

    def add_ra_scene(self, ra_scene):
        """
        :type ra_scene: _models.RaScene
        """
        for ra_scene_item in ra_scene.scene:
            ra_oid = ra_scene_item.oid

            ra_track_context = self._ra_oid_to_context_map.get(ra_oid, None)
            if ra_track_context:
                # Update existing ra_track
                # if not ra_track_context.rm_delayed_call.active():
                #    pass
                ra_track_context.rm_delayed_call.reset(self._ra_track_store_tout_sec)
                ra_track_context.ra_track.append(ra_scene_item)

            else:
                # Create new ra_track.
                ra_track = _models.RaTrack(ra_scene_item)

                # Schedule to remove ra_track after _ra_track_tout_sec since the last update.
                rm_delayed_call = self._twisted_reactor.callLater(
                    self._ra_track_store_tout_sec, self._handle_ra_track_tout, ra_oid)

                self._ra_oid_to_context_map[ra_oid] = _RaTrackContext(ra_track, rm_delayed_call)

    def _handle_ra_track_tout(self, ra_oid):
        ra_track_context = self._ra_oid_to_context_map.get(ra_oid, None)
        if not ra_track_context.rm_request:
            if self._ra_track_tout_handler:
                self._ra_track_tout_handler(ra_track_context.ra_track)

        self._ra_oid_to_context_map.pop(ra_oid, None)

    def remove_ra_track(self, ra_oid):
        """
        :type ra_oid: int
        """
        ra_track_context = self._ra_oid_to_context_map.get(ra_oid, None)
        if ra_track_context:
            # ra_track will be removed automaticaly after _ra_track_tout_sec timeout since the last update.
            # rm_request flag indicates that it's necessary to hide ra_track presence in the store for its clients.
            ra_track_context.rm_request = True

    def try_get_ra_track(self, ra_oid):
        """
        :type ra_oid: int
        :rtype: _models.RaTrack | None
        """
        ra_track_context = self._ra_oid_to_context_map.get(ra_oid, None)
        if ra_track_context and not ra_track_context.rm_request:
            return ra_track_context.ra_track

        return None
